package thesis.crawler;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;

import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;

import org.htmlcleaner.HtmlCleaner;
import org.htmlcleaner.TagNode;
import org.htmlcleaner.XPatherException;

public class GoogleCrawler {

	public final static String SEARCH_HOST_NAME = "www.google.com";
	public final static int SEARCH_HOST_PORT = 80;
	public final static int SEARCH_MAXIMUM_EACH_BENCH = 100;
	public final static int IN_LINK_FOR_THE_SAME_DOMAIN_MAXIMUN = 10;
	public final static int SLEEP_TIME = 5000;
	public final static int UNCORRECT_PAGE_CONTENT_SHRESHOLD = 4000;
	public final static String DEFAULT_DIR = "D:\\12_Master_thesis\\04_Experiment\\6_4_food_200_100\\html_files";
	public final static int SUCCESSFUL = 0;
	public final static int EXCEPTION = 5;
	public final static int POSSIBLE_UNCORRECT = 10;
	public final static int IN_LINK_MAXIMUM = 20;
	
	private static GoogleCrawler crawler;
	private String dataDir;
	
	private GoogleCrawler()
	{
		this.dataDir = DEFAULT_DIR;
		File directory = new File(this.dataDir);
		if (!directory.exists())
			directory.mkdir();
	}
	
	public static GoogleCrawler getInstance()
	{
		if (crawler == null)
			crawler = new GoogleCrawler();
		return crawler;
	}
	
	public void setDataDir(String dir)
	{
		File directory = null;
		if (!this.dataDir.equals(dir))
		{
			directory = new File(this.dataDir);
			directory.delete();
			
			this.dataDir = dir;
			directory = new File(this.dataDir);
			if (!directory.exists())
				directory.mkdir();
		}
	}
	
	public String getDataDir()
	{
		return this.dataDir;
	}
	
	public String getGoogleSearchResultPageForInlink(String query, int start, long realNum)
	{
		return null;
	}
	
	/**
	 * Retrieve the Google search result web page
	 * @param query The query string
	 * @param resultSize The number of Google Search results
	 * @return The HTML content of Google Search result web page
	 * @throws UnknownHostException
	 * @throws IOException
	 */
	public String getGoogleSearchResultPage(String query, int start, long realNum) throws UnknownHostException, IOException
	{
		StringBuilder realQuery = new StringBuilder(query);
		realQuery.append("&num=").append(String.valueOf(realNum)).append("&start=").append(String.valueOf(start));
		
        Socket s = new Socket(SEARCH_HOST_NAME, SEARCH_HOST_PORT);
        PrintStream p = new PrintStream(s.getOutputStream());
        p.print("GET /search?q=" + realQuery.toString() + " HTTP/1.1\r\n");
        p.print("User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1) Gecko/20061010 Firefox/2.0\r\n");
        p.print("Connection: close\r\n\r\n");
        
        InputStreamReader in = new InputStreamReader(s.getInputStream());
        BufferedReader buffer = new BufferedReader(in);
 
        String line;
        StringBuilder sb = new StringBuilder();
        
        while ((line = buffer.readLine()) != null)
            sb.append(line);
        
        in.close();
        
        return sb.toString();
	}
	
	public long getTotalSearchResult(String pageContent) throws IOException, XPatherException
	{
		HtmlCleaner cleaner = new HtmlCleaner();
		TagNode root = cleaner.clean(pageContent);
		
		Object[] objects = root.evaluateXPath("//div[@id='resultStats']");
		TagNode tagTotal = (TagNode)objects[0];
		//Object[] tagB = tagP.evaluateXPath("//b");
		//TagNode total = (TagNode)tagB[2];
		String text = tagTotal.getText().toString();
		String value[] = text.split(" ");
		
		try
		{
			String total = value[1];
			if (total.indexOf(",") != -1)
			{
				total = total.replace(",", "");
			}
			return Integer.valueOf(total);
		}
		catch (Exception e)
		{
			String total = value[0];
			if (total.indexOf(",") != -1)
			{
				total = total.replace(",", "");
			}
			return Integer.valueOf(total);
		}
	}
	
	private ArrayList<String> getURLsFromSearchResult(String pageContent) throws IOException, XPatherException
	{
		HtmlCleaner cleaner = new HtmlCleaner();
		TagNode root = cleaner.clean(pageContent);
		
		Object[] objects = root.evaluateXPath("//li[@class='g']");
		ArrayList<String> urlArray = new ArrayList<String>(objects.length);
		
		for (int i = 0; i < objects.length; i++)
		{
			TagNode li = (TagNode)objects[i];
			TagNode url = li.findElementByName("h3", true).findElementByName("a", true);
			
			// Discard the URL if the search result isn't normal search result
			if (checkResult(url.getText().toString()))
				continue;
			
			urlArray.add(url.getAttributeByName("href"));
		}
		
		return urlArray;
	}
	
	public HashMap<String, Integer> getInlinkDomains1(String desUrl, int inlinkNum) throws UnknownHostException, IOException, XPatherException, InterruptedException
	{
		String query = "link:" + desUrl;
		int start = 0, bench = 0;
		long total = 0, leftNum = 0;
		HashMap<String, Integer> map = new HashMap<String, Integer>();
		ArrayList<String> tempList = new ArrayList<String>();
		
		// Retrieve the first page of search result
		int firstTimeNum = (inlinkNum < SEARCH_MAXIMUM_EACH_BENCH) ? inlinkNum : SEARCH_MAXIMUM_EACH_BENCH;
		String firstpage = getGoogleSearchResultPage(query, start, firstTimeNum);
		bench++;
		start = bench * SEARCH_MAXIMUM_EACH_BENCH;
		
		// Calculate the total matched items
		total = getTotalSearchResult(firstpage);
		
		// Extract the in-link urls from the first page
		tempList = getURLsFromSearchResult(firstpage);
		leftNum = total - fun(tempList, map, inlinkNum);
				
		// Extrac the urls from result page
		while (map.size() < inlinkNum && leftNum > 0)
		{
			// Wait a period between each google search request
			Thread.sleep(SLEEP_TIME);
			
			int number = (int)((leftNum < SEARCH_MAXIMUM_EACH_BENCH) ? leftNum : SEARCH_MAXIMUM_EACH_BENCH);
			String content = getGoogleSearchResultPage(query, start, number);
			ArrayList<String> list1 = getURLsFromSearchResult(content);
			
			// Re-set all related parameters
			leftNum = leftNum - fun(list1, map, inlinkNum);
			bench++;
			start = bench * SEARCH_MAXIMUM_EACH_BENCH;
		}
		
		return map;
	}
	
	
	public long fun(ArrayList<String> list, HashMap<String, Integer> map, int inlinkNum) throws MalformedURLException
	{
		int addedNum = 0;
		//int addedNum1 = 0;
		boolean stop = false;
		
		for (int i = 0; i < list.size(); i++)
		{
			String domain = getDomain(list.get(i));
			if (!map.containsKey(domain))
			{
				if (stop)
					return addedNum;
					
				map.put(domain, new Integer(1));
				addedNum++;
				//addedNum1++;
				if (map.size() >= inlinkNum)
					stop = true;
			}
			else
			{
				Integer value = (Integer)map.get(domain);
				int intValue = value.intValue();
				if (intValue < IN_LINK_MAXIMUM)
				{
					intValue++;
					map.put(domain, new Integer(intValue));
					addedNum++;
					//addedNum1++;
				}
				else
				{
					addedNum++;
					continue;
				}
			}
		}
		return addedNum;
	}
	
    public HashMap<String, Integer> getInlinkDomains(String destUrl, int inlinkNum) throws UnknownHostException, IOException, XPatherException, InterruptedException
	{
		String query = "link:" + destUrl;
		int size = 0, start = 0, count = 0;
		HashMap<String, Integer> map = new HashMap<String, Integer>();
		ArrayList<String> tempList = new ArrayList<String>();
		
		long total = getTotalSearchResult(getGoogleSearchResultPage(query, start, SEARCH_MAXIMUM_EACH_BENCH));
		long realNum = total < inlinkNum ? total : inlinkNum;
		long bench = realNum/SEARCH_MAXIMUM_EACH_BENCH;
		
		// Wait some time between each Google search request
		Thread.sleep(SLEEP_TIME);
		
		// First extraction of in-links before compression
		if (realNum <= SEARCH_MAXIMUM_EACH_BENCH)
		{
			tempList = getURLsFromSearchResult(getGoogleSearchResultPage(query, start, realNum));
			// Wait some time between each Google search request
			Thread.sleep(SLEEP_TIME);
			count = addToInlinkMap(tempList, map, count, realNum);
		}
		else
		{
			for (int i = 0; i <= bench; i++)
			{
				start = SEARCH_MAXIMUM_EACH_BENCH * i;
				size = (int)((i != bench) ? SEARCH_MAXIMUM_EACH_BENCH : realNum - (i * SEARCH_MAXIMUM_EACH_BENCH));
				
				tempList = getURLsFromSearchResult(getGoogleSearchResultPage(query, start, size));
				// Wait some time between each Google search request
				Thread.sleep(SLEEP_TIME);
				count = addToInlinkMap(tempList, map, count, realNum);
			}
		}
		
		// The new in-links fill out phrase
		long left = total - bench * SEARCH_MAXIMUM_EACH_BENCH;
		while (count < realNum && left > 0)
		{
			start = (int) (bench * SEARCH_MAXIMUM_EACH_BENCH);
			size = (int)((left < SEARCH_MAXIMUM_EACH_BENCH) ? left : SEARCH_MAXIMUM_EACH_BENCH);
			
			tempList = getURLsFromSearchResult(getGoogleSearchResultPage(query, start, size));
			left -= bench * SEARCH_MAXIMUM_EACH_BENCH;
			// Wait some time between each Google search request
			Thread.sleep(SLEEP_TIME);
			count = addToInlinkMap(tempList, map, count, realNum);		
			bench++;
		}
		
		return map;
	}
	
	private int addToInlinkMap(ArrayList<String> srcList, HashMap<String, Integer> map, int count, long realNum) throws MalformedURLException
	{
		String domain = null;
		int number = count;
		
		for (int i = 0; i < srcList.size(); i++)
		{
			domain = getDomain(srcList.get(i));
			if (domain == null)
				continue;
			if (!map.containsKey(domain))
			{
				map.put(domain, new Integer(1));
				number++;
				if (number == realNum)
					break;
			}
			else
			{
				Integer value = (Integer)map.get(domain);
				int intValue = value.intValue();
				if (intValue >= IN_LINK_FOR_THE_SAME_DOMAIN_MAXIMUN)
					continue;
				else
				{
					intValue++;
					map.put(domain, new Integer(intValue));
					number++;
					if (number == realNum)
						break;
				}
			}
		}
		
		return number;
	}
	
	public ArrayList<String> getInlinkURLs(String destUrl, int inlinkNum) throws UnknownHostException, IOException, XPatherException, InterruptedException
	{
		String query = "link:" + destUrl;
		String page;
		
		int bench = inlinkNum/SEARCH_MAXIMUM_EACH_BENCH;
		int size = 0;
		int start = 0;
		
		ArrayList<String> urlList = new ArrayList<String>(inlinkNum);
		ArrayList<String> tempList = new ArrayList<String>(SEARCH_MAXIMUM_EACH_BENCH);
		
		// Get the original URLs
		for (int i = 0; i < bench; i++)
		{
			start = SEARCH_MAXIMUM_EACH_BENCH * i;
			if (inlinkNum < SEARCH_MAXIMUM_EACH_BENCH)
				size = inlinkNum;
			else
				size = (i != bench) ? SEARCH_MAXIMUM_EACH_BENCH : inlinkNum - (i * SEARCH_MAXIMUM_EACH_BENCH);
			
			page = getGoogleSearchResultPage(query, start, size);
			urlList.addAll(getURLsFromSearchResult(page));
			
			Thread.sleep(SLEEP_TIME);
		}
		
		// Compress the inlink array
		int num = compressInlinks(urlList, getDomain(urlList.get(0)), true);
		int total = num;
		String stand = getDomain(urlList.get(urlList.size() - 1));
		
		while (total != inlinkNum)
		{
			page = getGoogleSearchResultPage(query, bench, SEARCH_MAXIMUM_EACH_BENCH);
			tempList = getURLsFromSearchResult(page);
			
			Thread.sleep(SLEEP_TIME);
			
			num = compressInlinks(tempList, stand, false);
			
			if (total + num >= inlinkNum)
			{
				urlList.addAll(tempList.subList(0, inlinkNum - total));
				break;
			}
			else
			{
				urlList.addAll(tempList);
				total += num;
			}
			
			bench++;
			stand = getDomain(tempList.get(tempList.size() - 1));
		}
		
		return urlList;
	}
	
	public ArrayList<String> getResultURLs(String query, int number) throws UnknownHostException, IOException, XPatherException, InterruptedException
	{
		int bench = number/SEARCH_MAXIMUM_EACH_BENCH;
		int start = 0, size = 0;
		ArrayList<String> urlList = new ArrayList<String>(number);
		
		for (int i = 0; i <= bench; i++)
		{
			start = SEARCH_MAXIMUM_EACH_BENCH * i;
			if (number < SEARCH_MAXIMUM_EACH_BENCH)
				size = number;
			else
				size = (i != bench) ? SEARCH_MAXIMUM_EACH_BENCH : number - (i * SEARCH_MAXIMUM_EACH_BENCH);
			
			String content = getGoogleSearchResultPage(query, start, size);
			urlList.addAll(getURLsFromSearchResult(content));
			
			// Wait some time between each Google search request
			Thread.sleep(SLEEP_TIME);
		}
		
		return urlList;
	}

	private int compressInlinks(ArrayList<String> inlinks, String standInit, boolean firstTime) throws MalformedURLException
	{
		String stand = standInit;
		int count = 0;
		for (int i = firstTime ? 1 : 0; i < inlinks.size(); i++)
		{
			if (getDomain(inlinks.get(i)).equals(stand))
			{
				++count;
				if (count > IN_LINK_FOR_THE_SAME_DOMAIN_MAXIMUN)
				{
					inlinks.remove(i);
					i--;
				}
			}
			else
			{
				stand = getDomain(inlinks.get(i));
				count = 0;
			}
		}
		return inlinks.size();
	}
	
	public String getDomain(String urlStr) throws MalformedURLException
	{
		try
		{
			URL url = new URL(urlStr);
			return url.getHost();
		}
		catch (MalformedURLException e)
		{
//			int i = urlStr.indexOf("http://");
//			String sub = urlStr.substring(i);
//			URL url1 = new URL(urlStr.substring(urlStr.indexOf("http://")));
//			return url1.getHost();
			return null;
		}
		
	}
	
	/**
	 * Check if the result is normal search result or not
	 * @param str The short title of search result
	 * @return False if the result is normal search result, otherwise True
	 */
    private boolean checkResult(String str)
    {
    	if (str.contains("Video results for") 
    			|| str.contains("Image results for") 
    			|| str.contains("News results for") 
    			|| str.contains("Shopping results for") 
    			|| str.contains("Latest results for")
    			//|| str.contains("Images for") // This one is test code
    			)
    	{
    		return true;
    	}
    	return false;
    }
    
	/**
	 * Download the web page to specified directory
	 * @param directory The folder where to place the web page
	 * @param fileName The file name of the web page
	 * @param url The URL string of the web page
	 * @throws IOException
	 */
	public void downloadPage(String fileName, String url) throws IOException
	{
		FileWriter fwriter = new FileWriter(this.dataDir + "\\" + fileName);
		BufferedWriter bwriter = new BufferedWriter(fwriter);
		
		bwriter.write(getPageContent(url));
		bwriter.close();
		fwriter.close();
	}
	
	public int downloadPage(String url, int id)
	{
		try
		{
			FileWriter fwriter = new FileWriter(this.dataDir + "\\" + id + ".htm");
			BufferedWriter bwriter = new BufferedWriter(fwriter);
			
			// TEST CODE
			String text = "<self_url href=\"" + url + "\"></self_url>" + getPageContent(url);
			bwriter.write(text);
			
			//bwriter.write(getPageContent(url));
			bwriter.close();
			fwriter.close();
			if (text.length() < UNCORRECT_PAGE_CONTENT_SHRESHOLD)
				return POSSIBLE_UNCORRECT;
			return SUCCESSFUL;
		}
		catch (IOException e)
		{
			System.out.println(e.toString());
			return EXCEPTION;
		}
		finally
		{
			;
		}
	}
	
	public int downloadPage(String content, String url, int id)
	{
		try
		{
			FileWriter fwriter = new FileWriter(this.dataDir + "\\" + id + ".htm");
			BufferedWriter bwriter = new BufferedWriter(fwriter);
			
			// TEST CODE
			String text = "<self_url href=\"" + url + "\"></self_url>" + content;
			bwriter.write(text);
			
			//bwriter.write(getPageContent(url));
			bwriter.close();
			fwriter.close();
			if (text.length() < UNCORRECT_PAGE_CONTENT_SHRESHOLD)
				return POSSIBLE_UNCORRECT;
			return SUCCESSFUL;
		}
		catch (IOException e)
		{
			System.out.println(e.toString());
			return EXCEPTION;
		}
		finally
		{
			;
		}
	}
	
	/**
	 * Download a web page
	 * @param url The URL object of target web page to be downloaded
	 * @return The HTML content of target web page
	 * @throws IOException
	 */
	public String getPageContent(URL url) throws IOException
	{
		BufferedReader reader = new BufferedReader(new InputStreamReader(url.openStream()));
		
		String line;
		StringBuilder sb = new StringBuilder();
		
		while ((line = reader.readLine()) != null)
			sb.append(line);
		
		return sb.toString();
	}
	
	/**
	 * Download a web page
	 * @param urlStr The URL string of target web page to be downloaded
	 * @return The HTML content of target web page
	 * @throws IOException
	 */
	public String getPageContent(String urlStr) throws IOException
	{
		return getPageContent(new URL(urlStr));
	}
}
