package thesis.crawler;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class YahooCrawler {

	private final static String APP_ID = "CycLNg7V34FozgY966R2clsSRvaJ_Kppfj2Nq51xn4h.HLpuDM3WRNVW_3361Q--";
	private final static String SERVICE_URL = "http://search.yahooapis.com/SiteExplorerService/V1/inlinkData";
	private final static int SEARCH_MAXIMUM_EACH_BENCH = 100;
	private final static int IN_LINK_MAXIMUM = 10;
	private final static int SEARCH_LIMITATION = 1000;
	
	private static YahooCrawler instance;
	private YahooCrawler(){}
	
	public static YahooCrawler getInstance()
	{
		if (instance == null)
			instance = new YahooCrawler();
		return instance;
	}
	
	public HashMap<String, Integer> getInlinkDomains(String desUrl, int inlinkMaximum) throws IOException, SAXException, ParserConfigurationException
	{
		HashMap<String, Integer> map = new HashMap<String, Integer>();
		int startIndex = 1, bench = 0;
		ArrayList<String> tempList = new ArrayList<String>();
		
		int firstTimeNum = (inlinkMaximum < SEARCH_MAXIMUM_EACH_BENCH) ? inlinkMaximum : SEARCH_MAXIMUM_EACH_BENCH;
		String url = constructYahooQuery(desUrl, startIndex, firstTimeNum);
		String pageContent = getSearchPageContentXML(url);
		long leftNum = getTotalNumber(pageContent);
		bench++;
		startIndex = bench * SEARCH_MAXIMUM_EACH_BENCH + 1;
		
		tempList = getUrlsFromPageXML(pageContent);
		leftNum -= addToMap(map, tempList, inlinkMaximum);
		
		int limitation = startIndex + SEARCH_MAXIMUM_EACH_BENCH - 1;
		while (map.size() < inlinkMaximum && leftNum > 0 && limitation <= SEARCH_LIMITATION)
		{
			int number = (int)((leftNum < SEARCH_MAXIMUM_EACH_BENCH) ? leftNum : SEARCH_MAXIMUM_EACH_BENCH);
			String url1 = constructYahooQuery(desUrl, startIndex, number);
			String page = getSearchPageContentXML(url1);
			ArrayList<String> list1 = getUrlsFromPageXML(page);
			
			// Re-set all related parameters
			leftNum -= addToMap(map, list1, inlinkMaximum);
			bench++;
			startIndex = bench * SEARCH_MAXIMUM_EACH_BENCH + 1;
			limitation = startIndex + SEARCH_MAXIMUM_EACH_BENCH - 1;
		}
		
		return map;
	}
	
	public String getDomain(String urlStr) throws MalformedURLException
	{
		URL url = new URL(urlStr);
		return url.getHost();
	}
	
	public long addToMap(HashMap<String, Integer> map, ArrayList<String> list, int inlinkNum) throws MalformedURLException
	{
		int addedNum = 0;
		boolean stop = false;
		
		for (int i = 0; i < list.size(); i++)
		{
			String domain = getDomain(list.get(i));
			if (!map.containsKey(domain))
			{
				if (stop)
					return addedNum;
					
				map.put(domain, new Integer(1));
				addedNum++;
				if (map.size() >= inlinkNum)
					stop = true;
			}
			else
			{
				Integer value = (Integer)map.get(domain);
				int intValue = value.intValue();
				if (intValue < IN_LINK_MAXIMUM)
				{
					intValue++;
					map.put(domain, new Integer(intValue));
					addedNum++;
				}
				else
				{
					addedNum++;
					continue;
				}
			}
		}
		return addedNum;
	}
	
	public ArrayList<String> getUrlsFromPageXML(String xmlContent) throws ParserConfigurationException, SAXException, IOException
	{
		ArrayList<String> list = new ArrayList<String>();
		
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        InputSource is = new InputSource();
        is.setCharacterStream(new StringReader(xmlContent));

        Document doc = db.parse(is);
        NodeList nodes = doc.getElementsByTagName("Result");
        
        for (int i = 0; i < nodes.getLength(); i++)
        {
        	Element element = (Element) nodes.item(i);
        	NodeList urls = element.getElementsByTagName("Url");
        	Node url = urls.item(0);
        	list.add(url.getTextContent());
        }
        
		return list;
	}
	
	public long getTotalNumber(String xmlContent) throws ParserConfigurationException, SAXException, IOException
	{
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        InputSource is = new InputSource();
        is.setCharacterStream(new StringReader(xmlContent));

        Document doc = db.parse(is);
        NodeList nodes = doc.getElementsByTagName("ResultSet");
        Node root = nodes.item(0);
        NamedNodeMap attributes = root.getAttributes();
        Node totalAttr = attributes.getNamedItem("totalResultsAvailable");
        //totalAttr.getTextContent();
        return Long.parseLong(totalAttr.getNodeValue());
	}
	
	public String getSearchPageContentXML(String url) throws IOException
	{
		URL page = new URL(url);
		InputStream is = page.openStream();
		DataInputStream dis = new DataInputStream(new BufferedInputStream(is));

		StringBuilder sb = new StringBuilder();
		String s;
        while ((s = dis.readLine()) != null)
        	sb.append(s);
        
        return sb.toString();
	}
	
	public String constructYahooQuery(String desUrl, int startIndex, int number)
	{
		StringBuilder sb = new StringBuilder();
		
		// will omit the links that has the same domain as destination url
		sb.append(SERVICE_URL).append("?").append("appid=").append(APP_ID).append("&query=").append(desUrl)
		.append("&start=").append(String.valueOf(startIndex)).append("&results=").append(String.valueOf(number))
		.append("&omit_inlinks=").append("domain");
		
		return sb.toString();
	}
}
