package thesis.indexing;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.lucene.analysis.KeywordAnalyzer;
import org.apache.lucene.analysis.StopAnalyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;

import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.LockObtainFailedException;
import org.apache.lucene.util.Version;
import org.htmlcleaner.XPatherException;
import thesis.crawler.HtmlParser;

public class HtmlIndexer {

	private IndexWriter writer;
	
	public HtmlIndexer(String indexDir) throws CorruptIndexException, LockObtainFailedException, IOException
	{
		//writer = new IndexWriter(FSDirectory.open(new File(indexDir)), new StandardAnalyzer(Version.LUCENE_CURRENT), true, IndexWriter.MaxFieldLength.UNLIMITED);
		//writer = new IndexWriter(FSDirectory.open(new File(indexDir)), new KeywordAnalyzer(), true, IndexWriter.MaxFieldLength.UNLIMITED);
		writer = new IndexWriter(FSDirectory.open(new File(indexDir)), new StopAnalyzer(Version.LUCENE_CURRENT), true, IndexWriter.MaxFieldLength.UNLIMITED);
	}
	
	public void close() throws CorruptIndexException, IOException
	{
		writer.optimize();
		writer.close();
	}
	
	public int index(String dataDir) throws IOException, XPatherException
	{		
		File dir = new File(dataDir);
		File[] files = dir.listFiles();
		//File[] files = new File(dataDir).listFiles();
		
		System.out.println("The number of files to be indexed : " + files.length);
		for (int i = 0; i < files.length; i++)
		{
			File f = files[i];
			if (!f.isDirectory()
					&& !f.isHidden()
					&& f.exists()
					&& f.canRead())
			{
				indexFile(f);
				
				// TEST CODE
				System.out.println(f.getName() + " is indexed successfully!");
			}
		}
		return writer.numDocs();
	}
	
	private void indexFile(File file) throws CorruptIndexException, IOException, XPatherException
	{
		Document doc = getDocument(file);
		if (doc != null)
			writer.addDocument(doc);
	}
	
	private Document getDocument(File file) throws IOException, XPatherException
	{
		Document doc = new Document();
		StringBuilder sbuilder = new StringBuilder();
		BufferedReader input = new BufferedReader(new FileReader(file));
		String line = null;
		while ((line = input.readLine()) != null)
		{
			sbuilder.append(line);
			sbuilder.append(System.getProperty("line.separator"));
		}
		input.close();
		String content = sbuilder.toString();
		
		HtmlParser.getInstance().setPageContent(content);
		
		String url = HtmlParser.getInstance().getUrl();
		String title = HtmlParser.getInstance().getTitle();
		String description = HtmlParser.getInstance().getDescription();
		String contents = HtmlParser.getInstance().getPlainContent();
		//String pagerank = String.valueOf(HtmlParser.getInstance().getPageRank());
		
		String url1 = "", title1 = "", description1 = "", contents1 = "";
		//pagerank1 = "";
		if (url != null)
			url1 = url;
		if (title != null)
			title1 = title;
		if (description != null)
			description1 = description;
		if (contents != null)
			contents1 = contents;
		//if (pagerank != null)
			//pagerank1 = pagerank;
		
		StringBuilder sb = new StringBuilder();
		sb.append(title).append(description).append(contents);
		
		doc.add(new Field("url", url1, Field.Store.YES, Field.Index.NOT_ANALYZED));
		//doc.add(new Field("title", title1, Field.Store.YES, Field.Index.ANALYZED, Field.TermVector.YES));
		//doc.add(new Field("description", description1, Field.Store.YES, Field.Index.ANALYZED, Field.TermVector.YES));
		doc.add(new Field("content", sb.toString(), Field.Store.YES, Field.Index.ANALYZED, Field.TermVector.YES));
		//doc.add(new Field("pagerank", pagerank1, Field.Store.YES, Field.Index.NOT_ANALYZED));

		return doc;
	}
	
	public int index(ArrayList<String> searchResults) throws IOException, XPatherException
	{
		for (int i = 0; i < searchResults.size(); i++)
		{
			indexFile(searchResults.get(i));
			System.out.println(i + " Status[true] URL : " + searchResults.get(i));
		}
		return writer.numDocs();
	}
	
	private void indexFile(String url) throws CorruptIndexException, IOException, XPatherException
	{
		Document doc = getDocument(url);
		if (doc != null)
			writer.addDocument(doc);
	}
	
	private Document getDocument(String url)
	{
		// TODO
		return null;
//		Document doc = new Document();
//		String content = GoogleCrawler.getInstance().getPageContent(url);
//		
//		doc.add(new Field("url", url, Field.Store.YES, Field.Index.NOT_ANALYZED));
//		doc.add(new Field("title", HtmlParser.getInstance().getTitle(content), Field.Store.YES, Field.Index.ANALYZED));
//		doc.add(new Field("description", HtmlParser.getInstance().getDescription(content), Field.Store.YES, Field.Index.ANALYZED));
//		doc.add(new Field("content", HtmlParser.getInstance().getPlainContent(url), Field.Store.NO, Field.Index.ANALYZED));
//		doc.add(new Field("pagerank", String.valueOf(HtmlParser.getInstance().getPageRank(url)), Field.Store.YES, Field.Index.NOT_ANALYZED));
//		
//		return doc;
	}
	
}
