package thesis.indexing;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.LockObtainFailedException;
import org.apache.lucene.util.Version;
import org.htmlcleaner.XPatherException;

import thesis.crawler.HtmlParser;

public class TxtIndexer {

	private IndexWriter writer;
	
	public TxtIndexer(String indexDir) throws CorruptIndexException, LockObtainFailedException, IOException
	{
		writer = new IndexWriter(FSDirectory.open(new File(indexDir)), new StandardAnalyzer(Version.LUCENE_CURRENT), true, IndexWriter.MaxFieldLength.UNLIMITED); 
	}
	
	public void close() throws CorruptIndexException, IOException
	{
		writer.optimize();
		writer.close();
	}
	
	public int index(String dataDir) throws IOException, XPatherException
	{
		File[] files = new File(dataDir).listFiles();
		
		System.out.println("The number of files to be indexed : " + files.length);
		for (int i = 0; i < files.length; i++)
		{
			File f = files[i];
			if (!f.isDirectory()
					&& !f.isHidden()
					&& f.exists()
					&& f.canRead())
			{
				indexFile(f);
				
				// TEST CODE
				System.out.println(f.getName() + " is indexed successfully!");
			}
		}
		return writer.numDocs();
	}
	
	private void indexFile(File file) throws CorruptIndexException, IOException, XPatherException
	{
		Document doc = getDocument(file);
		if (doc != null)
			writer.addDocument(doc);
	}
	
	private Document getDocument(File file) throws IOException, XPatherException
	{
		Document doc = new Document();
		StringBuilder sbuilder = new StringBuilder();
		BufferedReader input = new BufferedReader(new FileReader(file));
		String line = null;
		while ((line = input.readLine()) != null)
		{
			sbuilder.append(line);
			sbuilder.append(System.getProperty("line.separator"));
		}
		input.close();
		String content = sbuilder.toString();
		
		doc.add(new Field("links", content, Field.Store.YES, Field.Index.ANALYZED, Field.TermVector.YES));
		return doc;
	}
}
