package thesis.main;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Map.Entry;

import org.htmlcleaner.XPatherException;

import thesis.crawler.GoogleCrawler;
import thesis.crawler.HtmlParser;
import thesis.crawler.YahooCrawler;
import thesis.vector.combination.PageRankSerializer;

public class Test_DownloadSearchResults {

	/**
	 * @param args
	 * @throws InterruptedException 
	 * @throws XPatherException 
	 * @throws IOException 
	 * @throws UnknownHostException 
	 */
	public static void main(String[] args) throws UnknownHostException, IOException, XPatherException, InterruptedException {
		int resultSize = 200;
		int inlinkNum = 100;
		String root = "D:\\12_Master_thesis\\04_Experiment\\";
		//=======================================================================
		String query = "food";
		String inlinkDir = root + "6_4_food_200_100\\inlink_files";
		String outlinkDir = root + "6_4_food_200_100\\outlink_files";
		String PRFile = root + "6_4_food_200_100\\pagerank_files\\pagerank.ser";
		//=======================================================================
		
		ArrayList<String> searchResults = GoogleCrawler.getInstance().getResultURLs(query, resultSize);
		ArrayList<String> pagerankList = new ArrayList<String>();
		
		int tag;
		int missed = 0, possible_error = 0;
		ArrayList<Integer> missedList = new ArrayList<Integer>();
		ArrayList<Integer> errorList = new ArrayList<Integer>();
		
		for (int i = 0; i < searchResults.size(); i++)
		{

			if (searchResults.get(i).endsWith(".pdf") 
					|| searchResults.get(i).endsWith(".doc") 
					|| searchResults.get(i).endsWith(".ppt")
					|| searchResults.get(i).endsWith(".jpg"))
			{
				System.out.println("[WARNING] " + i + " " + searchResults.get(i) + "is not a web page!");
				continue;
			}
			else
			{
				System.out.println("[INFO] " + i + " " + searchResults.get(i));
				// Get page content for each url
				//String content = GoogleCrawler.getInstance().getPageContent(searchResults.get(i));
			
				// 1 Construct html file for each url
				tag = GoogleCrawler.getInstance().downloadPage(searchResults.get(i), i);
				if (tag == GoogleCrawler.EXCEPTION) // Down load page unsuccessfully
				{
					missed++;
					missedList.add(new Integer(i));
					System.out.println("    Download html file...[MISSED]");
				}
				else // Down load page successfully
				{
					// 1 Retrieve the page rank
					int pagerank = HtmlParser.getInstance().getPageRank(searchResults.get(i));
					if (pagerank == 0)
						pagerank = HtmlParser.getInstance().getPageRank("http://" + GoogleCrawler.getInstance().getDomain(searchResults.get(i)));
					
					pagerankList.add(String.valueOf(pagerank));
					System.out.println("    PageRank = " + pagerank);
					if (tag == GoogleCrawler.POSSIBLE_UNCORRECT)
					{
						possible_error++;
						errorList.add(new Integer(i));
						System.out.println("    Download html file......[POSSIBLE UNCORRECT]");
					}
					else
						System.out.println("    Download html file......[OK]");
					
					// 2 Construct out-link file for each url
					HashMap<String, Integer> omap = HtmlParser.getInstance().getOutlinkDomains(searchResults.get(i));
					int outdegree = writeOutLinkFile(omap, outlinkDir, null, i);
					System.out.println("    Create out-links file...[OK] outdegree = " + outdegree);
					
					// 3 Construct in-link file for each url
					try
					{
						HashMap<String, Integer> imap = YahooCrawler.getInstance().getInlinkDomains(searchResults.get(i), inlinkNum);
						int indegree = writeInLinkFile(imap, inlinkDir, null, i);
						System.out.println("    Create in-links file....[OK] indegree = " + indegree);
					}
					catch (Exception e)
					{
						System.out.println("    Create in-links file....[FAILURE]");
						continue;
					}
				}
			}
		}
		
		// 4 Save PageRank file
		PageRankSerializer.serialize(pagerankList, PRFile);
		
		System.out.println();
		System.out.println("Completed!");
		System.out.println("Total number of pages to be downloaded : " + searchResults.size());
		System.out.println("Number of pages downloaded : " + (searchResults.size() - missed - possible_error));
		System.out.println("Number of pages missed : " + missed);
		System.out.println("Number of pages possible uncorrect :" + possible_error);
		System.out.println("List of missed pages :");
		for (int i = 0; i < missedList.size(); i++)
		{
			System.out.println(missedList.get(i).intValue());
		}	
		System.out.println("List of possible uncorrect pages :");
		for (int i = 0; i < errorList.size(); i++)
		{
			System.out.println(errorList.get(i).intValue());
		}
	}
	
	private static int writeToFile(HashMap<String, Integer> map, String dir, String url, int id) throws IOException
	{
		File directory = new File(dir);
		if (directory.isDirectory() && !directory.exists())
			directory.mkdir();
		
		String fileName = dir + "\\" + id + ".txt";
		FileWriter fwriter = new FileWriter(fileName);
		BufferedWriter bwriter = new BufferedWriter(fwriter);
		
		Set<Entry<String, Integer>> set = map.entrySet();
		Iterator<Entry<String, Integer>> it = set.iterator();
		
		int total = 0;
		while (it.hasNext())
		{
			Entry<String, Integer> me = it.next();
			for (int i = 0; i < me.getValue(); i++)
			{
				total++;
				bwriter.write(me.getKey());
				//bwriter.write(" ");
				bwriter.newLine();
				//bwriter.write("\n");
			}
		}
		
		bwriter.close();
		fwriter.close();
		return total;
	}
	
	public static int writeInLinkFile(HashMap<String, Integer> map, String inlinkDir, String url, int id) throws IOException
	{
		return writeToFile(map, inlinkDir, url, id);
	}

	public static int writeOutLinkFile(HashMap<String, Integer> map, String outlinkDir, String url, int id) throws IOException
	{
		return writeToFile(map, outlinkDir, url, id);
	}
}
