//package org.apache.mahout.common.distance;
package thesis.similarity;

import java.util.Collection;
import java.util.Collections;

import org.apache.hadoop.mapred.JobConf;
import org.apache.mahout.common.distance.CosineDistanceMeasure;
import org.apache.mahout.common.distance.DistanceMeasure;
import org.apache.mahout.common.parameters.Parameter;
import org.apache.mahout.math.CardinalityException;
import org.apache.mahout.math.RandomAccessSparseVector;
import org.apache.mahout.math.Vector;

public class WeightedDistanceMeasure implements DistanceMeasure {
	
	//jaguar
	private final static int TERM_VECTOR_DIMENSION = 4444;
	private final static int INLINK_VECTOR_DIMENSION = 1006;
	private final static int OUTLINK_VECTOR_DIMENSION = 1527;
	private final static int PAGERANK_VECTOR_DIMENSION = 11;
	
	// big apple
//	private final static int TERM_VECTOR_DIMENSION = 4825;
//	private final static int INLINK_VECTOR_DIMENSION = 502;
//	private final static int OUTLINK_VECTOR_DIMENSION = 2155;
//	private final static int PAGERANK_VECTOR_DIMENSION = 11;
	
	private final static double TERM_VECTOR_WEIGHT = 0;
	private final static double INLINK_VECTOR_WEIGHT = 0.5;
	private final static double OUTLINK_VECTOR_WEIGHT = 0.3;
	private final static double PAGERANK_VECTOR_WEIGHT = 0.2;

	@Override
	public double distance(Vector v1, Vector v2) {
		if (v1.size() != v2.size())
			throw new CardinalityException();
		
		double value = 0.0;
		// Re-construction of v1
		RandomAccessSparseVector t_v1 = new RandomAccessSparseVector(TERM_VECTOR_DIMENSION);
		RandomAccessSparseVector i_v1 = new RandomAccessSparseVector(INLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector o_v1 = new RandomAccessSparseVector(OUTLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector p_v1 = new RandomAccessSparseVector(PAGERANK_VECTOR_DIMENSION);	
		for (int i = 0, start = 0; i < TERM_VECTOR_DIMENSION; i++, start++)
		{
			value = v1.getQuick(start);
			if (value != 0.0)
			{
				t_v1.setQuick(i, value);
			}
		}
		t_v1.setName(v1.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION; i < INLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = v1.getQuick(start);
			if (value != 0.0)
			{
				i_v1.setQuick(i, value);
			}
		}
		i_v1.setName(v1.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION; i < OUTLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = v1.getQuick(start);
			if (value != 0.0)
			{
				o_v1.setQuick(i, value);
			}
		}
		o_v1.setName(v1.getName());
		
		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION + OUTLINK_VECTOR_DIMENSION; i < PAGERANK_VECTOR_DIMENSION; i++, start++)
		{
			value = v1.getQuick(start);
			if (value != 0.0)
			{
				p_v1.setQuick(i, value);
			}
		}
		p_v1.setName(v1.getName());

		// Re-construction of v2
		RandomAccessSparseVector t_v2 = new RandomAccessSparseVector(TERM_VECTOR_DIMENSION);
		RandomAccessSparseVector i_v2 = new RandomAccessSparseVector(INLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector o_v2 = new RandomAccessSparseVector(OUTLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector p_v2 = new RandomAccessSparseVector(PAGERANK_VECTOR_DIMENSION);
		for (int i = 0, start = 0; i < TERM_VECTOR_DIMENSION; i++, start++)
		{
			value = v2.getQuick(start);
			if (value != 0.0)
			{
				t_v2.setQuick(i, value);
			}
		}
		t_v2.setName(v2.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION; i < INLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = v2.getQuick(start);
			if (value != 0.0)
			{
				i_v2.setQuick(i, value);
			}
		}
		i_v2.setName(v2.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION; i < OUTLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = v2.getQuick(start);
			if (value != 0.0)
			{
				o_v2.setQuick(i, value);
			}
		}
		o_v2.setName(v2.getName());
		
		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION + OUTLINK_VECTOR_DIMENSION; i < PAGERANK_VECTOR_DIMENSION; i++, start++)
		{
			value = v2.getQuick(start);
			if (value != 0.0)
			{
				p_v2.setQuick(i, value);
			}
		}
		p_v2.setName(v2.getName());
		
		// calculate distance for each vector pair
		CosineDistanceMeasure cos = new CosineDistanceMeasure();
		double result = TERM_VECTOR_WEIGHT * cos.distance(t_v1, t_v2) 
						+ INLINK_VECTOR_WEIGHT * cos.distance(i_v1, i_v2) 
						+ OUTLINK_VECTOR_WEIGHT * cos.distance(o_v1, o_v2) 
						+ PAGERANK_VECTOR_WEIGHT * cos.distance(p_v1, p_v2);
		return result;
	}

	@Override
	public double distance(double centroidLengthSquare, Vector centroid, Vector v) {
		double value = 0.0;
		// Re-construction of v1
		RandomAccessSparseVector t_centroid = new RandomAccessSparseVector(TERM_VECTOR_DIMENSION);
		RandomAccessSparseVector i_centroid = new RandomAccessSparseVector(INLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector o_centroid = new RandomAccessSparseVector(OUTLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector p_centroid = new RandomAccessSparseVector(PAGERANK_VECTOR_DIMENSION);	
		for (int i = 0, start = 0; i < TERM_VECTOR_DIMENSION; i++, start++)
		{
			value = centroid.getQuick(start);
			if (value != 0.0)
			{
				t_centroid.setQuick(i, value);
			}
		}
		t_centroid.setName(centroid.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION; i < INLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = centroid.getQuick(start);
			if (value != 0.0)
			{
				i_centroid.setQuick(i, value);
			}
		}
		i_centroid.setName(centroid.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION; i < OUTLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = centroid.getQuick(start);
			if (value != 0.0)
			{
				o_centroid.setQuick(i, value);
			}
		}
		o_centroid.setName(centroid.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION + OUTLINK_VECTOR_DIMENSION; i < PAGERANK_VECTOR_DIMENSION; i++, start++)
		{
			value = centroid.getQuick(start);
			if (value != 0.0)
			{
				p_centroid.setQuick(i, value);
			}
		}
		p_centroid.setName(centroid.getName());
		
		// Re-construction of v2
		RandomAccessSparseVector t_v = new RandomAccessSparseVector(TERM_VECTOR_DIMENSION);
		RandomAccessSparseVector i_v = new RandomAccessSparseVector(INLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector o_v = new RandomAccessSparseVector(OUTLINK_VECTOR_DIMENSION);
		RandomAccessSparseVector p_v = new RandomAccessSparseVector(PAGERANK_VECTOR_DIMENSION);
		for (int i = 0, start = 0; i < TERM_VECTOR_DIMENSION; i++, start++)
		{
			value = v.getQuick(start);
			if (value != 0.0)
			{
				t_v.setQuick(i, value);
			}
		}
		t_v.setName(v.getName());

		for (int i = 0, start = TERM_VECTOR_DIMENSION; i < INLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = v.getQuick(start);
			if (value != 0.0)
			{
				i_v.setQuick(i, value);
			}
		}
		i_v.setName(v.getName());
		
		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION; i < OUTLINK_VECTOR_DIMENSION; i++, start++)
		{
			value = v.getQuick(start);
			if (value != 0.0)
			{
				o_v.setQuick(i, value);
			}
		}
		o_v.setName(v.getName());
		
		for (int i = 0, start = TERM_VECTOR_DIMENSION + INLINK_VECTOR_DIMENSION + OUTLINK_VECTOR_DIMENSION; i < PAGERANK_VECTOR_DIMENSION; i++, start++)
		{
			value = v.getQuick(start);
			if (value != 0.0)
			{
				p_v.setQuick(i, value);
			}
		}
		p_v.setName(v.getName());
	
		CosineDistanceMeasure cos = new CosineDistanceMeasure();
		double result = TERM_VECTOR_WEIGHT * cos.distance(t_centroid.getLengthSquared(), t_centroid, t_v) 
		                + INLINK_VECTOR_WEIGHT * cos.distance(i_centroid.getLengthSquared(), i_centroid, i_v)
		                + OUTLINK_VECTOR_WEIGHT * cos.distance(o_centroid.getLengthSquared(), o_centroid, o_v)
		                + PAGERANK_VECTOR_WEIGHT * cos.distance(p_centroid.getLengthSquared(), p_centroid, p_v);
		return result;
	}

	@Override
	public void createParameters(String arg0, JobConf arg1) {
		// nothing to do
	}

	@Override
	public Collection<Parameter<?>> getParameters() {
		return Collections.emptyList();
	}

	@Override
	public void configure(JobConf arg0) {
		// nothing to do
	}

}
